# -*- coding: utf-8 -*-
"""SSH YAML Configuration

NAME - SSH YAML Configuration
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT
"""

# Underscore Variables

"""Title of Program"""
__title__ = 'sshyc'
"""Version of Program"""
__version__ = '1.0.2'
"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'

# Imports

from sshyc.app import main

# Running as sshyc command

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""SSH YAML Configuration

NAME - SSH YAML Configuration
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT
"""
# Imports

from sshyc.app import main

# Underscore Variables

"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'

# Main Script
main()



# -*- coding: utf-8 -*-
"""SSH YAML Configuration

NAME - SSH YAML Configuration
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT
"""
# Imports

from sshyc import __version__
import os
import yaml
from typing import Dict

# Underscore Variables

"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'

# Variables


debug = True

# Lib Functions


def log(level: str = None, *args) -> None:
    if level is None:
        print('SSHYC:', *args)
    elif level == "DEBUG":
        if debug:
            print('SSHYC [{0}]:'.format(level), *args)
    else:
        print('SSHYC [{0}]:'.format(level), *args)


def read_yaml(filename: str) -> dict:
    """Function to read data from yaml file
    Args:
        filename (object): File Name

    Returns:
        :obj:`str`: Returning Data from YAML file
    """
    dane = {}
    if os.path.isfile(filename):
        with open(filename, "r") as plik:
            dane = yaml.full_load(plik)
    return dane


def special_yaml_config_tags(key, value) -> Dict[str, str]:
    generic_key = str(key).lower()
    if generic_key in ('$proxy'):
        value_split = value.split(':')
        if len(value_split) >= 1:
            host = value_split[0]
            if len(value_split) >= 2:
                port = value_split[1]
            else:
                port = "22"
        else:
            log('SYNTAX_WARNING', 'Invalid $Proxy value')
            return {}
        return {
            "ProxyCommand": "ssh -q -p {0} -W %h:%p {1}".format(port, host),
        }
    return {key: value}

# Main App Function


def main() -> None:
    print("SSH YAML Configuration v.{0}".format(__version__))

    # Variables
    config_path = os.path.expanduser('~/.ssh/config')

    # Check if config files exist
    file_paths = [
        os.path.expanduser('~/.ssh/config.yaml'),
        os.path.expanduser('~/.ssh/config.yml'),
    ]
    found_path = None

    for file_path in file_paths:
        log('DEBUG', 'Checking configuration file path:', file_path)
        if os.path.isfile(file_path):
            found_path = file_path
            log('DEBUG', 'Configuration File found in:', found_path)
            break
    log('DEBUG', 'Configuration checked')
    if found_path is None:
        log('ERROR', 'Make configuration file in any of these locations:')
        for file_path in file_paths:
            log('ERROR', '   -', file_path)
        log('ERROR', 'and run tool again')
        return

    configuration = read_yaml(found_path)

    if configuration is None:
        log('ERROR', 'Configuration file in', found_path, 'is empty. Create configuration and run tool again')
        return

    generated_file = ""
    generated_yaml = {}

    for root_key, root_value in configuration.items():
        # all key
        if str(root_key).lower() in ('all', '*'):
            generated_yaml['*'] = {}
            if root_value is not None and isinstance(root_value, dict):
                for key, value in root_value.items():
                    generated_yaml['*'].update(special_yaml_config_tags(key, value))
            else:
                log('SYNTAX_WARNING', 'Invalid {0} value'.format(root_key))
        # hosts key
        if str(root_key).lower() in ('hosts'):
            if root_value is not None and isinstance(root_value, dict):
                for host_key, host_value in root_value.items():
                    generated_yaml[host_key] = {}
                    if root_value is not None and isinstance(host_value, dict):
                        for key, value in host_value.items():
                            generated_yaml[host_key].update(special_yaml_config_tags(key, value))
                    else:
                        log('SYNTAX_WARNING', 'Invalid {0}.{1} value'.format(root_key, host_key))
            else:
                log('SYNTAX_WARNING', 'Invalid {0} value'.format(root_key))
        # groups key
        if str(root_key).lower() in ('groups'):
            if root_value is not None and isinstance(root_value, dict):
                for group_key, group_value in root_value.items():
                    if root_value is not None and isinstance(group_value, dict):
                        for host_key, host_value in group_value.items():
                            generated_yaml[host_key] = {}
                            if root_value is not None and isinstance(host_value, dict):
                                for key, value in host_value.items():
                                    generated_yaml[host_key].update(special_yaml_config_tags(key, value))
                            else:
                                log('SYNTAX_WARNING', 'Invalid {0}.{1}.{2} value'.format(root_key, group_key, host_key))
                    else:
                        log('SYNTAX_WARNING', 'Invalid {0}.{1} value'.format(root_key, group_key))
            else:
                log('SYNTAX_WARNING', 'Invalid {0} value'.format(root_key))
        # projects key
        if str(root_key).lower() in ('projects'):
            if root_value is not None and isinstance(root_value, dict):
                for project_key, project_value in root_value.items():
                    if root_value is not None and isinstance(project_value, dict):
                        for host_key, host_value in project_value.items():
                            generated_yaml[host_key] = {}
                            if root_value is not None and isinstance(host_value, dict):
                                for key, value in host_value.items():
                                    generated_yaml[host_key].update(special_yaml_config_tags(key, value))
                            else:
                                log('SYNTAX_WARNING', 'Invalid {0}.{1}.{2} value'.format(root_key, project_key, host_key))
                    else:
                        log('SYNTAX_WARNING', 'Invalid {0}.{1} value'.format(root_key, project_key))
            else:
                log('SYNTAX_WARNING', 'Invalid {0} value'.format(root_key))
        # hosts groups key
        if str(root_key).lower() in ('host_groups', 'hosts_groups'):
            if root_value is not None and isinstance(root_value, dict):
                for group_key, group_value in root_value.items():
                    if root_value is not None and isinstance(group_value, dict):
                        for host_key, host_value in group_value.items():
                            host = "{0}.{1}".format(host_key, group_key)
                            generated_yaml[host] = {}
                            if root_value is not None and isinstance(host_value, dict):
                                for key, value in host_value.items():
                                    generated_yaml[host].update(special_yaml_config_tags(key, value))
                            else:
                                log('SYNTAX_WARNING', 'Invalid {0}.{1}.{2} value'.format(root_key, group_key, host_key))
                    else:
                        log('SYNTAX_WARNING', 'Invalid {0}.{1} value'.format(root_key, group_key))
            else:
                log('SYNTAX_WARNING', 'Invalid {0} value'.format(root_key))
    # print(yaml.dump(generated_yaml))
    # print(generated_yaml)

    padding = "     "
    eol = "\n"
    for host_name, host_config in generated_yaml.items():
        if host_config is not None and isinstance(host_config, dict):
            generated_conf = "Host {0}{1}".format(host_name, eol)
            for config_key, config_value in host_config.items():
                generated_conf += "{0}{1} {2}{3}".format(padding, config_key, config_value, eol)
            generated_file += generated_conf
        else:
            log('WARNING', 'Host {0} has empty or invalid configuration'.format(host_name))
    # print(generated_file)

    log(None, 'Generating SSH Config File')
    with open(config_path, "w") as plik:
        plik.write(generated_file)
    # f = open(file_path, 'r', encoding='utf-8')
    # log



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""The setup script."""

# Imports
import io
from setuptools import setup, find_packages

# Readme file
with io.open('README.rst', encoding='utf-8') as readme_file:
    readme = readme_file.read()
# ChangeLog file
with io.open('HISTORY.rst', encoding='utf-8') as history_file:
    history = history_file.read()
# Requirements Variable
requirements: list = [
    # Package Requirements
    'pyyaml',
]
# Setup Requirements Variable
setup_requirements: list = [
    # Setup Requirements
]
# Test Requirements Variable
test_requirements: list = [
    # Test Requirements
]
setup(
    # Name of Package
    name='sshyc',

    # Version following SemVer Style
    version='1.0.2',

    # Description of the Package
    description='SSHYC short for SSH YAML Configuration is tool to make .ssh/config from YAML files.',

    # Description of the Package to show on PyPi (Longer Description)
    long_description=readme + '\n\n' + history,

    # The Project Main Page [For that project for now is just repository]
    url='https://github.com/paip-web/sshyc',

    # Author Details
    author='Patryk Adamczyk',
    author_email='patrykadamczyk@paipweb.com',

    # License
    license='MIT',

    # Classifiers of the Project
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        # 'Development Status :: 1 - Planning'
        # 'Development Status :: 2 - Pre-Alpha'
        # 'Development Status :: 3 - Alpha'
        # 'Development Status :: 4 - Beta'
        # 'Development Status :: 5 - Production/Stable'
        # 'Development Status :: 6 - Mature'
        # 'Development Status :: 7 - Inactive'
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',

        # Topic
        'Topic :: Utilities',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Operating System :: OS Independent',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Microsoft :: Windows :: Windows 7',
        'Operating System :: Microsoft :: Windows :: Windows 10',
        'Operating System :: POSIX :: Linux',
        'Environment :: Console'
    ],

    # Keywords of your Project
    keywords='ssh tools ssh_config config yaml',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # Dependencies of the Project
    install_requires=requirements,
    tests_require=test_requirements,
    setup_requires=setup_requirements,

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'setup': ["wheel", "twine", "collective.checkdocs", "bumpversion"],
        # 'test': ['pylint', 'pytest', 'coverage'],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    # package_data={
    #    'sample': ['package_data.dat'],
    # },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'sshyc=sshyc:main',
        ],
    },
    # Python Required Version for the package
    python_requires='~=3.6',
)
